@@ -11,13 +11,17 @@ from collections import Counter
 from typing import TYPE_CHECKING
 
 from pyxform import aliases, constants
+from pyxform.constants import EXTERNAL_INSTANCE_EXTENSIONS, ROW_FORMAT_STRING
 from pyxform.errors import PyXFormError
 from pyxform.utils import default_is_dynamic, is_valid_xml_tag, levenshtein_distance
-from pyxform.xls2json_backends import csv_to_dict, xls_to_dict
+from pyxform.validators.pyxform import select_from_file_params
+from pyxform.validators.pyxform.missing_translations_check import (
+    missing_translations_check,
+)
+from pyxform.xls2json_backends import csv_to_dict, xls_to_dict, xlsx_to_dict
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, KeysView, Optional
-
+    from typing import Any, Dict, KeysView, List, Optional
 
 SMART_QUOTES = {"\u2018": "'", "\u2019": "'", "\u201c": '"', "\u201d": '"'}
 _MSG_SUPPRESS_SPELLING = (
@@ -92,11 +96,11 @@ def replace_smart_quotes_in_dict(_d):
 
 
 def dealias_and_group_headers(
-    dict_array,
-    header_aliases,
-    use_double_colons,
-    default_language="default",
-    ignore_case=False,
+    dict_array: "List[Dict]",
+    header_aliases: "Dict",
+    use_double_colons: bool,
+    default_language: str = constants.DEFAULT_LANGUAGE_VALUE,
+    ignore_case: bool = False,
 ):
     """
     For each row in the worksheet, group all keys that contain a double colon.
@@ -220,7 +224,7 @@ def group_dictionaries_by_key(list_of_dicts, key, remove_key=True):
     return dict_of_lists
 
 
-def has_double_colon(workbook_dict):
+def has_double_colon(workbook_dict) -> bool:
     """
     Look for a column header with a doublecolon (::) and
     return true if one is found.
@@ -344,7 +348,7 @@ def workbook_to_json(
     workbook_dict,
     form_name=None,
     fallback_form_name=None,
-    default_language="default",
+    default_language=constants.DEFAULT_LANGUAGE_VALUE,
     warnings=None,
 ) -> "Dict[str, Any]":
     """
@@ -389,8 +393,6 @@ def workbook_to_json(
             "The survey sheet is either empty or missing important column headers."
         )
 
-    row_format_string = "[row : %s]"
-
     # Make sure the passed in vars are unicode
     form_name = str(form_name)
     default_language = str(default_language)
@@ -438,7 +440,7 @@ def workbook_to_json(
     settings = settings_sheet[0] if len(settings_sheet) > 0 else {}
     replace_smart_quotes_in_dict(settings)
 
-    default_language = settings.get(constants.DEFAULT_LANGUAGE, default_language)
+    default_language = settings.get(constants.DEFAULT_LANGUAGE_KEY, default_language)
 
     # add_none_option is a boolean that when true,
     # indicates a none option should automatically be added to selects.
@@ -457,7 +459,7 @@ def workbook_to_json(
         constants.TITLE: id_string,
         constants.ID_STRING: id_string,
         constants.SMS_KEYWORD: sms_keyword,
-        constants.DEFAULT_LANGUAGE: default_language,
+        constants.DEFAULT_LANGUAGE_KEY: default_language,
         # By default the version is based on the date and time yyyymmddhh
         # Leaving default version out for now since it might cause
         # problems for formhub.
@@ -487,6 +489,8 @@ def workbook_to_json(
         choices_sheet, aliases.list_header, use_double_colons, default_language
     )
     combined_lists = group_dictionaries_by_key(choices_sheet, constants.LIST_NAME)
+    # To combine the warning into one message, the check for missing choices translation
+    # columns is run with Survey sheet below.
 
     choices = combined_lists
     # Make sure all the options have the required properties:
@@ -568,6 +572,14 @@ def workbook_to_json(
     )
     survey_sheet = dealias_types(survey_sheet)
 
+    # Check for missing translations. The choices sheet is checked here so that the
+    # warning can be combined into one message.
+    warnings = missing_translations_check(
+        survey_sheet=survey_sheet,
+        choices_sheet=choices_sheet,
+        warnings=warnings,
+    )
+
     # No spell check for OSM sheet (infrequently used, many spurious matches).
     osm_sheet = dealias_and_group_headers(
         workbook_dict.get(constants.OSM, []), aliases.list_header, True
@@ -627,7 +639,7 @@ def workbook_to_json(
         # so the attributes below can be disabled.
         if "disabled" in row:
             warnings.append(
-                row_format_string % row_number
+                ROW_FORMAT_STRING % row_number
                 + " The 'disabled' column header is not part of the current"
                 + " spec. We recommend using relevant instead."
             )
@@ -648,13 +660,13 @@ def workbook_to_json(
             # then its a comment row, and we skip it with warning
             if not ((constants.NAME in row) or (constants.LABEL in row)):
                 warnings.append(
-                    row_format_string % row_number
+                    ROW_FORMAT_STRING % row_number
                     + " Row without name, text, or label is being skipped:\n"
                     + str(row)
                 )
                 continue
             raise PyXFormError(
-                row_format_string % row_number + " Question with no type.\n" + str(row)
+                ROW_FORMAT_STRING % row_number + " Question with no type.\n" + str(row)
             )
 
         # Pull out questions that will go in meta block
@@ -662,7 +674,7 @@ def workbook_to_json(
             # Force audit name to always be "audit" to follow XForms spec
             if "name" in row and row["name"] not in [None, "", "audit"]:
                 raise PyXFormError(
-                    row_format_string % row_number
+                    ROW_FORMAT_STRING % row_number
                     + " Audits must always be named 'audit.'"
                     + " The name column should be left blank."
                 )
@@ -831,11 +843,11 @@ def workbook_to_json(
                 question_default and default_is_dynamic(question_default, question_type)
             ):
                 raise PyXFormError(
-                    row_format_string % row_number + " Missing calculation."
+                    ROW_FORMAT_STRING % row_number + " Missing calculation."
                 )
         if question_type in constants.DEPRECATED_DEVICE_ID_METADATA_FIELDS:
             warnings.append(
-                (row_format_string % row_number)
+                (ROW_FORMAT_STRING % row_number)
                 + " "
                 + question_type
                 + " is no longer supported on most devices. "
@@ -858,7 +870,7 @@ def workbook_to_json(
                 control_name = question_name
                 if prev_control_type != control_type or len(stack) == 1:
                     raise PyXFormError(
-                        row_format_string % row_number
+                        ROW_FORMAT_STRING % row_number
                         + " Unmatched end statement. Previous control type: "
                         + str(prev_control_type)
                         + ", Control type: "
@@ -880,13 +892,13 @@ def workbook_to_json(
             #     row['name'] = "generated_group_name_" + str(row_number)
             else:
                 raise PyXFormError(
-                    row_format_string % row_number + " Question or group with no name."
+                    ROW_FORMAT_STRING % row_number + " Question or group with no name."
                 )
         question_name = str(row[constants.NAME])
         if not is_valid_xml_tag(question_name):
             if isinstance(question_name, bytes):
                 question_name = question_name.encode("utf-8")
-            error_message = row_format_string % row_number
+            error_message = ROW_FORMAT_STRING % row_number
             error_message += " Invalid question name [" + question_name + "] "
             error_message += "Names must begin with a letter, colon," + " or underscore."
             error_message += (
@@ -932,7 +944,7 @@ def workbook_to_json(
                     # Also means the error message text is stable for tests.
                     msg_dict = {"name": row.get("name"), "type": row.get("type")}
                     warnings.append(
-                        row_format_string % row_number
+                        ROW_FORMAT_STRING % row_number
                         + " %s has no label: " % control_type.capitalize()
                         + str(msg_dict)
                     )
@@ -945,13 +957,13 @@ def workbook_to_json(
                     if not parse_dict.get("list_name"):
                         # TODO: Perhaps warn and make repeat into a group?
                         raise PyXFormError(
-                            row_format_string % row_number
+                            ROW_FORMAT_STRING % row_number
                             + " Repeat loop without list name."
                         )
                     list_name = parse_dict["list_name"]
                     if list_name not in choices:
                         raise PyXFormError(
-                            row_format_string % row_number
+                            ROW_FORMAT_STRING % row_number
                             + " List name not in columns sheet: "
                             + list_name
                         )
@@ -1031,7 +1043,7 @@ def workbook_to_json(
                 select_type = aliases.select[parse_dict["select_command"]]
                 if select_type == "select one external" and "choice_filter" not in row:
                     warnings.append(
-                        row_format_string % row_number
+                        ROW_FORMAT_STRING % row_number
                         + " select one external is only meant for"
                         " filtered selects."
                     )
@@ -1053,14 +1065,14 @@ def workbook_to_json(
                             " 'list name', and 'name'."
                         )
                     raise PyXFormError(
-                        row_format_string % row_number
+                        ROW_FORMAT_STRING % row_number
                         + "List name not in external choices sheet: "
                         + list_name
                     )
                 if (
                     list_name not in choices
                     and select_type != "select one external"
-                    and file_extension not in [".csv", ".xml"]
+                    and file_extension not in EXTERNAL_INSTANCE_EXTENSIONS
                     and not re.match(r"\$\{(.*?)\}", list_name)
                 ):
                     if not choices:
@@ -1074,7 +1086,7 @@ def workbook_to_json(
                             " mandatory columns 'list_name', 'name', and 'label'."
                         )
                     raise PyXFormError(
-                        row_format_string % row_number
+                        ROW_FORMAT_STRING % row_number
                         + " List name not in choices sheet: "
                         + list_name
                     )
@@ -1083,7 +1095,7 @@ def workbook_to_json(
                 # they have no spaces (will cause errors in exports).
                 if (
                     select_type == constants.SELECT_ALL_THAT_APPLY
-                    and file_extension not in [".csv", ".xml"]
+                    and file_extension not in EXTERNAL_INSTANCE_EXTENSIONS
                 ):
                     for choice in choices[list_name]:
                         if " " in choice[constants.NAME]:
@@ -1103,9 +1115,16 @@ def workbook_to_json(
                 new_json_dict = row.copy()
                 new_json_dict[constants.TYPE] = select_type
 
-                # Look at parameters column for randomization parameters
+                select_params_allowed = ["randomize", "seed"]
+                if parse_dict["select_command"] in (
+                    "select_one_from_file",
+                    "select_multiple_from_file",
+                ):
+                    select_params_allowed += ["value", "label"]
+
+                # Look at parameters column for select parameters
                 parameters = get_parameters(
-                    row.get("parameters", ""), ["randomize", "seed"]
+                    row.get("parameters", ""), select_params_allowed
                 )
 
                 if "randomize" in parameters.keys():
@@ -1131,6 +1150,19 @@ def workbook_to_json(
                         "Parameters must include randomize=true to use a seed."
                     )
 
+                if "value" in parameters.keys():
+                    select_from_file_params.value_or_label_check(
+                        name="value",
+                        value=parameters["value"],
+                        row_number=row_number,
+                    )
+                if "label" in parameters.keys():
+                    select_from_file_params.value_or_label_check(
+                        name="label",
+                        value=parameters["label"],
+                        row_number=row_number,
+                    )
+
                 new_json_dict["parameters"] = parameters
 
                 if row.get("choice_filter"):
@@ -1147,7 +1179,7 @@ def workbook_to_json(
                 ):
                     new_json_dict["itemset"] = list_name
                     json_dict["choices"] = choices
-                elif file_extension in [".csv", ".xml"] or re.match(
+                elif file_extension in EXTERNAL_INSTANCE_EXTENSIONS or re.match(
                     r"\$\{(.*?)\}", list_name
                 ):
                     new_json_dict["itemset"] = list_name
@@ -1174,7 +1206,7 @@ def workbook_to_json(
                         parent_children_array.append(table_list_header)
 
                     if table_list != list_name:
-                        error_message = row_format_string % row_number
+                        error_message = ROW_FORMAT_STRING % row_number
                         error_message += (
                             " Badly formatted table list,"
                             " list names don't match: " + table_list + " vs. " + list_name
@@ -1229,7 +1261,7 @@ def workbook_to_json(
                 new_dict["bind"].update({"orx:max-pixels": parameters["max-pixels"]})
             else:
                 warnings.append(
-                    (row_format_string % row_number)
+                    (ROW_FORMAT_STRING % row_number)
                     + " Use the max-pixels parameter to speed up submission sending and save storage space. Learn more: https://xlsform.org/#image"
                 )
             parent_children_array.append(new_dict)
@@ -1274,9 +1306,16 @@ def workbook_to_json(
 
         if question_type in ["geopoint", "geoshape", "geotrace"]:
             new_dict = row.copy()
-            parameters = get_parameters(
-                row.get("parameters", ""), ["allow-mock-accuracy"]
-            )
+
+            if question_type == "geopoint":
+                parameters = get_parameters(
+                    row.get("parameters", ""),
+                    ["allow-mock-accuracy", "capture-accuracy", "warning-accuracy"],
+                )
+            else:
+                parameters = get_parameters(
+                    row.get("parameters", ""), ["allow-mock-accuracy"]
+                )
 
             if "allow-mock-accuracy" in parameters.keys():
                 if parameters["allow-mock-accuracy"] not in ["true", "false"]:
@@ -1287,6 +1326,29 @@ def workbook_to_json(
                     {"odk:allow-mock-accuracy": parameters["allow-mock-accuracy"]}
                 )
 
+            new_dict["control"] = new_dict.get("control", {})
+            if "capture-accuracy" in parameters.keys():
+                try:
+                    float(parameters["capture-accuracy"])
+                    new_dict["control"].update(
+                        {"accuracyThreshold": parameters["capture-accuracy"]}
+                    )
+                except ValueError:
+                    raise PyXFormError(
+                        "Parameter capture-accuracy must have a numeric value"
+                    )
+
+            if "warning-accuracy" in parameters.keys():
+                try:
+                    float(parameters["warning-accuracy"])
+                    new_dict["control"].update(
+                        {"unacceptableAccuracyThreshold": parameters["warning-accuracy"]}
+                    )
+                except ValueError:
+                    raise PyXFormError(
+                        "Parameter warning-accuracy must have a numeric value"
+                    )
+
             parent_children_array.append(new_dict)
             continue
 
@@ -1362,8 +1424,10 @@ def parse_file_to_workbook_dict(path, file_object=None):
     if not extension:
         raise PyXFormError("No extension.")
 
-    if extension in constants.SUPPORTED_FILE_EXTENSIONS:
+    if extension in constants.XLS_EXTENSIONS:
         return xls_to_dict(file_object if file_object is not None else path)
+    elif extension in constants.XLSX_EXTENSIONS:
+        return xlsx_to_dict(file_object if file_object is not None else path)
     elif extension == ".csv":
         return csv_to_dict(file_object if file_object is not None else path)
     else:
@@ -1380,7 +1444,7 @@ def get_filename(path):
 def parse_file_to_json(
     path,
     default_name="data",
-    default_language="default",
+    default_language=constants.DEFAULT_LANGUAGE_VALUE,
     warnings=None,
     file_object=None,
 ):
@@ -1512,7 +1576,10 @@ class QuestionTypesReader(SpreadsheetReader):
         types_sheet = "question types"
         self._dict = self._dict[types_sheet]
         self._dict = dealias_and_group_headers(
-            self._dict, {}, use_double_colons, "default"
+            dict_array=self._dict,
+            header_aliases={},
+            use_double_colons=use_double_colons,
+            default_language=constants.DEFAULT_LANGUAGE_VALUE,
         )
         self._dict = organize_by_values(self._dict, "name")
 
